/*
 * Scripts for Cambridge Dictionaries Online
 * Updated on: December 17, 2022
 * 
 * Copyright (C) 2016-2022  BLKSerene
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/* Tabs */
function add_tabs(dict, dicts) {
  var lang = dict.attr('class').substr(10);
  var cambridge_dicts = dict.find(dicts);

  if (cambridge_dicts.length > 1 && !dict.find('.tabs_cambridge').length) {
    var html_tabs = '<div class="tabs_cambridge tabs_cambridge_' + lang + '">'

    cambridge_dicts.each(function(i) {
      if (i > -1) {
        var $this = $(this);
        if (
          ($this.prev().prev('#dataset_caldcns').length) ||
          ($this.attr('data-id') && $this.attr('data-id').indexOf('cald4') > -1)
        ) {
          html_tabs += '<span class="tab_cambridge tab_cambridge_cald">Advanced Learner</span>';
        } else if ($this.attr('data-id') == 'cacd') {
          html_tabs += '<span class="tab_cambridge tab_cambridge_cacd">American</span>';
        } else if ($this.attr('data-id') == 'cbed') {
          html_tabs += '<span class="tab_cambridge tab_cambridge_cacd">Business</span>';
        } else if ($this.attr('data-id') && $this.attr('data-id').indexOf('examples') > -1) {
          html_tabs += '<span class="tab_cambridge tab_cambridge_examples">Examples</span>';
        /* cf. apple */
        } else if ($this.attr('data-id') == 'combinations') {
          html_tabs += '<span class="tab_cambridge tab_cambridge_collocations">Collocations</span>';
        /* Bilingual dictionaries */
        } else if ($this.children('.cid').length && $this.children('.cid').attr('id').indexOf('pw') > -1) {
          html_tabs += '<span class="tab_cambridge tab_cambridge_password">PASSWORD</span>';
        } else if ($this.children('.cid').length && $this.children('.cid').attr('id').indexOf('global') > -1) {
          html_tabs += '<span class="tab_cambridge tab_cambridge_global">GLOBAL</span>';
        /* English-Italian */
        } else if ($this.children('.cid').length && $this.children('.cid').attr('id').indexOf('ceed') > -1) {
          html_tabs += '<span class="tab_cambridge tab_cambridge_ceed">Essential</span>';
        /* English-Polish */
        } else if ($this.children('.cid').length && $this.children('.cid').attr('id').indexOf('cld') > -1) {
          html_tabs += '<span class="tab_cambridge tab_cambridge_ceed">Learner</span>';
        /* English-Portuguese/Spanish */
        } else if ($this.children('.cid').length && $this.children('.cid').attr('id').indexOf('cald') > -1) {
          html_tabs += '<span class="tab_cambridge tab_cambridge_ceed">Advanced Learner</span>';
        /* Default */
        } else {
          cid = $this.children('.cid').attr('id');

          if (cid) {
            html_tabs += '<span class="tab_cambridge tab_cambridge_' + cid + '">' + cid + '</span>';
          } else {
            html_tabs += '<span class="tab_cambridge tab_cambridge_dict">Dictionary</span>';
          }
        }
      }
    });

    dict.prepend(html_tabs + '</div>');

    var tabs_cambridge = dict.find('.tabs_cambridge .tab_cambridge');

    tabs_cambridge.each(function(i) {
      $(this).on('click', function() {
        tabs_cambridge.removeClass('in');
        $(this).addClass('in');

        cambridge_dicts.hide();
        cambridge_dicts.eq(i).show();
      });
    });

    tabs_cambridge.eq(0).click();
  }
}

function add_contents(dict, dicts, senses, wrapper) {
  var lang = dict.attr('class').substr(10);

  dict.find(dicts).each(function(i) {
    $this = $(this);
    $senses = $this.find(senses);

    if ($senses.length > 1 && !$this.find('.contents').length) {
      $this = $(this);

      $this.prepend('<a name="contents_' + lang + '_' + i + '"></a><div class="contents"><div class="contents_head">Contents</div><div class="contents_body"></div></div>');

      var contents = $this.find('.contents');
      var contentsHead = $this.find('.contents_head');
      var contentsBody = $this.find('.contents_body');

      // Slide contents
      contentsHead.on('click', function() {
        contentsHead.toggleClass('in');
        contentsBody.slideToggle();
      });

      contents.hide();
      contents.fadeIn();

      $senses.each(function(j) {
        var $this = $(this);
        
        // Headwords
        if ($this.closest('.phrase-block').find('.phrase-head').length) {
          // Phrases
          var headword = $this.closest('.phrase-block').find('.phrase-title').text();
        } else if ($this.closest('.runon').length) {
          // Derivative
          var headword = $this.closest('.runon').find('.runon-title').text();
        } else {
          var headword = $this.closest(wrapper).find('.di-title').text();
        }

        // POS
        if ($this.closest('.phrase-block').length) {
          // Phrases
          var pos = 'phrase';
        } else if ($this.closest('.runon').length) {
          // Derivative
          if ($this.closest('.runon').find('.pos').eq(0).next().hasClass('gram')) {
            var pos = $this.closest('.runon').find('.pos').eq(0).text() + ' ' + $this.closest('.runon').find('.pos').eq(0).next().text();
          } else {
            var pos = $this.closest('.runon').find('.pos').eq(0).text();
          }
        } else if ($this.closest(wrapper).find('.anc-info-head').length) {
          // Phrasal verbs
          var pos = 'phrasal verb';
        } else if ($this.closest('.idiom-block').length) {
          // Idioms
          var pos = 'idiom';
        } else {
          if ($this.closest(wrapper).find('.pos').eq(0).next().hasClass('gram')) {
            var pos = $this.closest(wrapper).find('.pos').eq(0).text() + ' ' + $this.closest(wrapper).find('.pos').eq(0).next().text();
          } else {
            var pos = $this.closest(wrapper).find('.pos').eq(0).text();
          }
        }

        headword = headword.trim()
        pos = pos.trim()

        // Blue bars
        if ($this.closest('.dsense').find('.dsense_h').length && !$this.closest('.phrase-block').length) {
          var senseHeadings = $this.closest('.dsense').find('.dsense_h').eq(0);
          var text = '. ' + headword + ' · ' + pos + ' ' + senseHeadings.children('.guideword').text().trim();
        } else if (pos) {
          var text = '. ' + headword + ' · ' + pos;
        } else {
          var text = '. ' + headword;
        }

        if ($this.closest('.dsense').find('.dsense_h').length) {
          var senseHeadings = $this.closest('.dsense').find('.dsense_h').eq(0);

          if (!senseHeadings.parent('a').length) {
            senseHeadings.wrap($('<a href="#contents_' + lang + '_' + i + '"></a>'));
          }
        }

        contentsBody.append('<a href="#contents_item_' + lang + '_' + i + '_' + j + '" class="contents_item">' + (j + 1) + text + '</a>');
        $this.before('<a name="contents_item_' + lang + '_' + i + '_' + j + '"></a>');
      });
    }
  });
}

$(document).ready(function() {
  // Fix links to "Labels & Codes"
  var helpLabelCodes = 'https://dictionary.cambridge.org' + $('.gc, .gcs').parent('a').attr('href')

  $('.gc, .gcs').each(function() {
    var element_a = $(this).parent('a');

    if (element_a.length && element_a.attr('href')[0] == '/') {
      $(this).parent('a').attr('href', helpLabelCodes);
    }
  });

  // Fix links in Thesaurus
  $('.daccord.fs16 a').each(function() {
    href = $(this).attr('href')

    if (href[0] == '/') {
      $(this).attr('href', 'https://dictionary.cambridge.org' + href);
    }
  });
  
  // Sliding items
  $('.daccord_h, .ca_h').off().on('click', function() {
    var $this = $(this);
      
    $this.toggleClass('in');
    $this.next().slideToggle();
  });

  /* Generate contents */
  $('.cambridge_eng_zho_cn').each(function() {
    var dict = $(this);
    
    var dicts = '.superentry .di-body, .superentry .dataset';
    var senses = '.def-block';
    var wrapper = '.entry-body__el, .idiom-block';

    add_contents(dict, dicts, senses, wrapper);
    add_tabs(dict, dicts);
  });

  $('.cambridge_eng_eng').each(function() {
    var dict = $(this);
    
    var dicts = '.page .dictionary, .page .dataset';
    var senses = '.def-block';
    var wrapper = '.entry-body__el, .idiom-block';

    add_contents(dict, dicts, senses, wrapper);
    add_tabs(dict, dicts);
  });

  $('.cambridge_eng_fra, .cambridge_eng_deu, .cambridge_eng_ind').each(function() {
    var dict = $(this);
      
    var dicts = '.entry-body .dictionary, .entry-body .dataset';
    var senses = '.def-block';
    var wrapper = '.kdic';

    add_contents(dict, dicts, senses, wrapper);
    add_tabs(dict, dicts);
  });

  $('.cambridge_eng_ita, .cambridge_eng_jpn, .cambridge_eng_pol, .cambridge_eng_por, .cambridge_eng_spa').each(function() {
    var dict = $(this);

    var dicts = '.page .dictionary, .page .dataset';
    var senses = '.def-block';
    var wrapper = '.entry-body__el';
    
    add_contents(dict, dicts, senses, wrapper);
    add_tabs(dict, dicts);
  });

  var dicts_to_eng = [
    '.cambridge_fra_eng',
    '.cambridge_deu_eng',
    '.cambridge_ind_eng',
    '.cambridge_ita_eng',
    '.cambridge_jpn_eng',
    '.cambridge_pol_eng',
    '.cambridge_por_eng',
    '.cambridge_spa_eng'
  ];

  $(dicts_to_eng.join(', ')).each(function() {
    var dict = $(this);
      
    var dicts = '.entry-body .dictionary, .entry-body .dataset';
    var senses = '.def-block';
    var wrapper = '.kdic';

    add_contents(dict, dicts, senses, wrapper);
    add_tabs(dict, dicts);
  });

  $('.cambridge_eng_ara').each(function() {
    var dict = $(this);
    
    var senses = '.def-block';
    var wrapper = '.entry-body__el';
    var dicts = '.entry-body .di, .entry-body .dataset';

    add_contents(dict, dicts, senses, wrapper);
    add_tabs(dict, dicts)
  });
});
